import $ from "jquery"
import Timer from "easytimer.js"

import alertify from "alertifyjs";
import "alertifyjs/build/css/alertify.css";

const disableInputs = () => {
    $("#pause_play,#minutes,#seconds,#update_time").prop("disabled", true)
}

const enableInputs = () => {
    $("#pause_play,#minutes,#seconds,#update_time").prop("disabled", false)
}

const updateTimeText = (timer) => {
    const timeValues = timer.getTimeValues()

    const hours = String(timeValues.minutes + timeValues.hours * 60 + timeValues.days * 1440).padStart(2, '0')
    const seconds = String(timeValues.seconds).padStart(2, '0')

    $('#timer .timer-values').html(`${hours}:${seconds}`)
}

const assignEventListeners = (timer) => {
    timer.addEventListener(
        'secondsUpdated',
        (event) => {
            updateTimeText(timer)
        }
    )

    timer.addEventListener(
        'targetAchieved',
        (event) => {
            $("#is_counting").prop("checked", false)
            $('#timer .timer-values').html("Konec")
        }
    )

    updateTimeText(timer)
}

const updateTimer = (timer, data, continuePlaying) => {
    console.info(`Updating timer: ${JSON.stringify(data.sync_time)}`)

    window.startingTime = {
        minutes: data["sync_time"]["minutes"],
        seconds: data["sync_time"]["seconds"]
    }

    timer.removeAllEventListeners()

    timer = new Timer({
        countdown: true,
        startValues: {
            minutes: window.startingTime.minutes,
            seconds: window.startingTime.seconds,
        }
    })

    assignEventListeners(timer)

    if (window.timerIsRunning) {
        timer.start()
    }

    return timer
}

const syncTime = (timerSocket, timer) => {
    timerSocket.send(JSON.stringify({
        "sync": timer.getTimeValues()
    }))
}

$(window).ready(
    () => {
        disableInputs()

        // --- BEGIN Timer ---

        window.timerIsRunning = false

        let timer = new Timer({
            countdown: true,
            startValues: {
                minutes: window.startingTime.minutes,
                seconds: window.startingTime.seconds,
            }
        })

        let timerSocket = null
        let isInitialConnect = true

        const connectToSocket = () => {
            timerSocket = new WebSocket(
                (
                    (window.location.protocol === "https:") ?
                    "wss://" : "ws://"
                )
                + window.location.host
                + "/ws/timer/"
                + window.timerId
                + "/"
            )

            if (!isInitialConnect) {
                alertify.success("Obnovování spojení.")
            }

            isInitialConnect = false

            timerSocket.onmessage = (event) => {
                enableInputs()

                console.info("Received timer message:", event.data)

                const data = JSON.parse(event.data)

                if ("sync_time" in data) {
                    // Only update if there is any real difference.

                    const remainingTime = timer.getTimeValues()

                    if (
                        data.sync_time.minutes !== remainingTime.minutes ||
                        data.sync_time.seconds !== remainingTime.seconds
                    ) {
                        timer = updateTimer(timer, data)
                    }
                }

                if ("is_running" in data) {
                    if (data["is_running"]) {
                        // Reset if we are playing again.
                        const remainingTime = timer.getTimeValues()

                        if (remainingTime.seconds === 0 && remainingTime.minutes === 0) {
                            timer = new Timer({
                                countdown: true,
                                startValues: {
                                    minutes: window.startingTime.minutes,
                                    seconds: window.startingTime.seconds,
                                }
                            })

                            assignEventListeners(timer)
                        }

                        timer.start()

                        $("#is_counting").prop("checked", true)
                        $("#pause_play > .btn__body").html("⏸︎")
                        window.timerIsRunning = true
                    } else {
                        timer.pause()

                        $("#is_counting").prop("checked", false)
                        $("#pause_play > .btn__body").html("⏵︎")
                        window.timerIsRunning = false
                    }
                }
            }

            let interval = null

            timerSocket.onopen = () => {
                syncTime(timerSocket, timer)

                interval = setInterval(syncTime, 5000, timerSocket, timer)

                // --- BEGIN Controls ---

                $("#pause_play").on(
                    "click",
                    (event) => {
                        $("#is_counting").click()
                    }
                )

                $("#is_counting").on(
                    "change",
                    (event) => {
                        disableInputs()

                        if (event.target.checked) {
                            console.info("Starting timer")

                            $("#pause_play > .btn__body").html("⏸︎")
                            timerSocket.send(JSON.stringify({
                                "is_running": true
                            }))
                        } else {
                            console.info("Stopping timer")

                            $("#pause_play > .btn__body").html("⏵︎")
                            timerSocket.send(JSON.stringify({
                                "is_running": false
                            }))
                        }
                    }
                )

                $("#update_time").on(
                    "click",
                    (event) => {
                        disableInputs()

                        timer.pause()

                        let minutes = Number($("#minutes").val())
                        let seconds = Number($("#seconds").val())

                        timerSocket.send(JSON.stringify({
                            "time": {
                                "minutes": minutes,
                                "seconds": seconds
                            },
                            "is_running": false
                        }))

                        $("#is_counting").prop("checked", false)
                    }
                )

                // --- END Controls ---
            }

            timerSocket.onclose = (event) => {
                disableInputs()

                alertify.error("Ztráta spojení, pokoušíme se o zpětné připojení.")

                setTimeout(connectToSocket, 5000)
                clearInterval(interval)
                $("#is_counting,#pause_play,#update_time").unbind("click")
            }
        }

        connectToSocket()
        assignEventListeners(timer)

        // --- END Timer ---
    }
)
